.weather-current-summarybox {
padding-top:5px;
}

.weather-current-summary {
font-size: 31px;
}

.weather-current-databox {
padding: 5px 15px 0px;
}

.weather-current-icon {
padding-right: 30px;
}

.weather-current-iconbox {
padding-right: 20px;
}

.weather-icon {
padding-right: 5px;
}

.weather-icon-rtl {
padding-left: 5px;
}

.weather-current-databox-captions {
text-align: right;
padding-right: 5px;
color: #999999;
}

.weather-forecast-icon {
padding-right: 15px;
}

.weather-forecasts {
-st-hfade-offset: 60px;
}

.weather-forecast-box {
}

.weather-forecast-databox {
min-width: 140px;
}

.weather-forecast-day {
color: #999999;
font-size: 90%;
}

.weather-forecast-iconbox {
}

.weather-sunrise-icon {
padding-right: 5px;
}

.weather-sunset-icon {
padding-left: 15px;
padding-right: 5px;
}

.weather-build-icon {
padding-left: 15px;
padding-right: 5px;
}
